<?php

/*
 * Personal Mommy
 * This Plugin is for players who did´nt find an end to play. Set a time and a reason,
 * why you have to go and your personal Mommy does it best and let you know what you
 * have to do.
 * Too long time ignoring your personal Mommy results in kick. :D
 *
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.9.2
 * Date:	2010-07-07
 * Copyright:	2010 by undef.de
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.06 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 */

/* The following manialink id's are used in this plugin (the 917 part of id can be changed on trouble in mommy_onStartUp()):
 * 91700		id for manialinks
 * 91701		id for manialink Helpwindow
 * 91702		id for action close Helpwindow
 */

Aseco::registerEvent('onStartup',			'mommy_onStartUp');
Aseco::registerEvent('onEverySecond',			'mommy_onEverySecond');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'mommy_onPlayerManialinkPageAnswer');

Aseco::addChatCommand('mommy',				'Setup a time to leave, see "/mommy help" for details.');

global $mommy_config;

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function mommy_onStartUp ($aseco) {
	global $mommy_config;

	if ( version_compare(XASECO_VERSION, 1.06, '<') ) {
		trigger_error('[plugin.personal_mommy.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.06!', E_USER_ERROR);
	}

	// Some settings
	$mommy_config['MANIALINK_ID'] = '917';				// Set internal Manialink ID
	$mommy_config['REFRESH_COUNT'] = 0;
	$mommy_config['MOMMY'] = '{#server}>> [Mommy] ';
	$mommy_config['VERSION'] = '0.9.2';
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_mommy ($aseco, $command) {
	global $mommy_config;

	$player = $aseco->server->players->getPlayer($command['author']->login);

	// Check optional parameter
	if (strtoupper($command['params']) == 'HELP') {
		mommy_help($aseco, $player, true);
	}
	elseif (strtoupper($command['params']) == 'OFF') {
		if ( isset($player->data['MOMMY']) ) {
			unset($player->data['MOMMY']);
			$message = $mommy_config['MOMMY'] .'{#server}Your personal Mommy is deactivated!';
		}
		else {
			$message = $mommy_config['MOMMY'] .'{#server}Your personal Mommy was not activated!';
		}
	}
	elseif (strtoupper($command['params']) == 'STATUS') {
		if ( isset($player->data['MOMMY']) ) {
			$message = $mommy_config['MOMMY'] .'{#server}Your personal Mommy start at '. date('H:i', $player->data['MOMMY']['TIMESTAMP']) .' (server time).';
		}
		else {
			$message = $mommy_config['MOMMY'] .'{#server}Your personal Mommy is not activated!';
		}
	}
	elseif ( preg_match("/^([0-9]+)(hour|min) (.*)$/i", $command['params']) ) {
		$param = preg_split("/^([0-9]+)(hour|min) (.*)$/i", $command['params'], 0, PREG_SPLIT_NO_EMPTY|PREG_SPLIT_DELIM_CAPTURE);

		$param[0] = (int)$param[0];
		if ( (($param[0] >= 1) && ($param[0] <= 59)) || (($param[0] >= 1) && ($param[0] <= 24)) ) {
			// Convert given time to seconds
			if (strtoupper($param[1]) == 'MIN') {
				$param[0] = $param[0] * 60;
			}
			elseif (strtoupper($param[1]) == 'HOUR') {
				$param[0] = $param[0] * 60 * 60;
			}

			// http://en.tm-wiki.org/wiki/Text_Formatting
			$param[2] = preg_replace('/^(\{#[A-Z]+\}|\$[A-F0-9TISWNMGZOHLP\$]+)$/i', '$1', $param[2]);		// remove > or >> from message or all colors and formating codes

			$player->data['MOMMY']['TIMESTAMP'] = time() + $param[0];
			$player->data['MOMMY']['REASON'] = $param[2];
			$player->data['MOMMY']['ACTION'] = 0;

			$message = $mommy_config['MOMMY'] .'{#server}Your personal Mommy is activated!';
		}
		else {
			$message = $mommy_config['MOMMY'] .'{#server}Wrong parameter for me! See {#highlite}/mommy help{#server} for details.';
		}
	}
	else {
		$message = $mommy_config['MOMMY'] .'{#server}Wrong parameter for me! See {#highlite}/mommy help{#server} for details.';
	}

	// Show message
	if ( isset($message) ) {
		if ( function_exists('send_window_message') ) {
			send_window_message($aseco, $message, $player);
		}
		else {
			$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $player->login);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function mommy_help ($aseco, $player, $display = true) {
	global $mommy_config;

	// Set the message
	$message = array(
		'This Plugin is for players who did\'nt find an end to play. Set a time and a reason, why you have to go and your personal Mommy',
		'does it best and let you know what you have to do.',
		'',
		'Too long time ignoring your personal Mommy results in kick. :D',
		'',
		'Possible parameters are:',
		'   $09F[1-24]hour $0F3[reason]',
		'   $09F[1-59]min $0F3[reason]',
		'',
		'Examples:',
		'   $FF0/mommy $09F2hour $0F3it is time to sleep now!',
		'   $FF0/mommy $09F5min $0F3it is time to sleep now!',
		'',
		'If you want to change your settings, just type the $FF0/mommy$Z command with other parameter.',
		'',
		'If you want to turn your personal Mommy off, just type:',
		'   $FF0/mommy off',
		'',
		'If you want to see at which time your personal Mommy are start to worry you, type:',
		'   $FF0/mommy status'
	);

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $mommy_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $mommy_config['MANIALINK_ID'] .'03">';

	if ($display == true) {
		// Window
		$xml .= '<frame posn="-40.1 30.45 -3">';	// BEGIN: Window Frame
		$xml .= '<quad posn="0.8 -0.8 0.01" sizen="78.4 53.7" bgcolor="000B"/>';
		$xml .= '<quad posn="-0.2 0.2 0.09" sizen="80.4 55.7" style="Bgs1InRace" substyle="BgCard3"/>';

		// Header Line
		$xml .= '<quad posn="0.8 -1.3 0.02" sizen="78.4 3" bgcolor="09FC"/>';
		$xml .= '<quad posn="0.8 -4.3 0.03" sizen="78.4 0.1" bgcolor="FFF9"/>';
		$xml .= '<quad posn="1.8 -1.4 0.10" sizen="2.8 2.8" style="Icons128x128_1" substyle="Buddies"/>';
		$xml .= '<label posn="5.5 -1.8 0.10" sizen="74 0" halign="left" textsize="2" scale="0.9" textcolor="FFFF" text="$L[http://www.undef.de/Trackmania/Plugins/]Personal Mommy/'. $mommy_config['VERSION'] .'$L"/>';

		// Close Button
		$xml .= '<frame posn="77.4 1.3 0">';
		$xml .= '<quad posn="0 0 0.10" sizen="4 4" style="Icons64x64_1" substyle="ArrowDown"/>';
		$xml .= '<quad posn="1.1 -1.35 0.11" sizen="1.8 1.75" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.65 -0.7 0.12" sizen="2.6 2.6" action="'. $mommy_config['MANIALINK_ID'] .'02" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';

		// Set the width of MessageWindow
		$width = 75;
		$line_height = 1.65;
		$position = 0;

		$xml .= '<frame posn="3 -6 0">';
		foreach ($message as $msg) {
			if ($msg) {
				$xml .= '<label posn="0 '. $position .' 0.05" sizen="'. ($width-2.6) .' 0" halign="left" textsize="1" textcolor="FFFF" text="'. $msg .'"/>';
			}
			$position -= $line_height;
		}
		$xml .= '</frame>';


		$xml .= '</frame>';	// Window
	}

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	$aseco->client->addCall("SendDisplayManialinkPageToLogin", array($player->login, $xml, 0, false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function mommy_onPlayerManialinkPageAnswer ($aseco, $answer) {
	global $mommy_config;

	// If id = 0, bail out immediately
	if ($answer[2] == 0) {
		return;
	}

	// Get Player
	$player = $aseco->server->players->getPlayer($answer[1]);

	if ($answer[2] == $mommy_config['MANIALINK_ID'] .'02') {		// Close help
		mommy_help($aseco, $player, false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Check every 30 seconds what action has to do for each player.
function mommy_onEverySecond ($aseco) {
	global $mommy_config;

	if ($mommy_config['REFRESH_COUNT'] >= 30) {

		// Reset RefreshCount
		$mommy_config['REFRESH_COUNT'] = 1;

		// Let mommy work
		mommy_action($aseco);
	}
	else {
		$mommy_config['REFRESH_COUNT'] += 1;		// Count up RefreshCount
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function mommy_action ($aseco) {
	global $mommy_config;

	foreach ($aseco->server->players->player_list as &$player) {
		if ( isset($player->data['MOMMY']) ) {
			if ( $player->data['MOMMY']['TIMESTAMP'] <= time() ) {

				switch ($player->data['MOMMY']['ACTION']) {
					case 0:
						$player->data['MOMMY']['ACTION'] ++;
						$player->data['MOMMY']['TIMESTAMP'] += 59;
						$message = $mommy_config['MOMMY'] . $player->nickname .'$Z{#server} you have to go now, because '. $player->data['MOMMY']['REASON'];
						break;
					case 1:
						$player->data['MOMMY']['ACTION'] ++;
						$player->data['MOMMY']['TIMESTAMP'] += 59;
						$message =$mommy_config['MOMMY'] . $player->nickname .'$Z{#server} you really have to go now, because '. $player->data['MOMMY']['REASON'];
						break;
					case 2:
						$player->data['MOMMY']['ACTION'] ++;
						$player->data['MOMMY']['TIMESTAMP'] += 29;
						$message = $mommy_config['MOMMY'] . $player->nickname .'$Z{#server} what do you do? End up NOW!';
						break;
					case 3:
						$player->data['MOMMY']['ACTION'] ++;
						$player->data['MOMMY']['TIMESTAMP'] += 29;
						$message = $mommy_config['MOMMY'] . $player->nickname .'$Z{#server} I´m very angry with you! Turn off this Game NOW!';
						break;
					case 4:
						$player->data['MOMMY']['ACTION'] ++;
						$player->data['MOMMY']['TIMESTAMP'] += 29;
						$message = $mommy_config['MOMMY'] . '{#server}Hey '. $player->nickname .'$Z{#server}! Last chance!';
						break;
					case 5:
						$message = $mommy_config['MOMMY'] . '{#server}Say bye bye '. $player->nickname .'$Z{#server}, because '. $player->data['MOMMY']['REASON'] .LF;
						$message .= $mommy_config['MOMMY'] . '{#server} kicked '. $player->nickname .'$Z{#server} from Server.';
						$aseco->client->query('Kick', $player->login);	// kick the player
						break;
				}

				if ( function_exists('send_window_message') ) {
					send_window_message($aseco, $message, false);
				}
				else {
					$aseco->client->query('ChatSendServerMessage', $aseco->formatColors($message));
				}
			}
		}
	}
}

?>
